@@ -622,12 +622,30 @@ class SoftwareProperties(object):
   def update_interface(self):
     " abstract interface to keep the UI alive "
 
+  def expand_http_line(self, line):
+    """
+    short cut - this:
+      apt-add-repository http://packages.medibuntu.org free non-free
+    same as
+      apt-add-repository 'deb http://packages.medibuntu.org/ '$(lsb_release -cs)' free non-free'
+    """
+    if not line.startswith("http"):
+      return line
+    repo = line.split()[0]
+    try:
+        areas = line.split(" ",1)[1]
+    except IndexError, e:
+        areas = "main"
+    line = "deb %s %s %s" % ( repo, self.distro.codename, areas )
+    return line
+
   def add_source_from_line(self, line):
     """
     Add a source with the given apt line and auto-add
     signing key if we have it in the whitelist
     """
     (deb_line, file) = expand_ppa_line(line.strip(), self.distro.codename)
+    deb_line = self.expand_http_line(deb_line)
     debsrc_line = 'deb-src' + deb_line[3:]
     new_deb_entry = SourceEntry(deb_line, file)
     new_debsrc_entry = SourceEntry(debsrc_line, file)
