@@ -30,7 +30,8 @@ import os
 import re
 from xml.sax.saxutils import escape
 
-import gtk
+from gi.repository import Gdk
+from gi.repository import Gtk
 import gobject
 
 from SimpleGtkbuilderApp import SimpleGtkbuilderApp
@@ -68,10 +69,10 @@ RESPONSE_ADD = 2
 
 def error(parent_window, summary, msg):
   " show a error dialog "
-  dialog = gtk.MessageDialog(parent=parent_window,
-                             flags=gtk.DIALOG_MODAL,
-                             type=gtk.MESSAGE_ERROR,
-                             buttons=gtk.BUTTONS_OK,
+  dialog = Gtk.MessageDialog(parent=parent_window,
+                             flags=Gtk.DialogFlags.MODAL,
+                             type=Gtk.MessageType.ERROR,
+                             buttons=Gtk.ButtonsType.OK,
                              message_format=None)
   dialog.set_markup("<big><b>%s</b></big>\n\n%s" % (summary, msg))
   res = dialog.run()
@@ -84,13 +85,13 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGtkbuilderApp):
         the used software repositories, corresponding authentication keys
         and update automation """
     SoftwareProperties.__init__(self, options=options, datadir=datadir)
-    gtk.window_set_default_icon_name("software-properties")
+    Gtk.Window.set_default_icon_name("software-properties")
 
     SimpleGtkbuilderApp.__init__(self, os.path.join(datadir, "gtkbuilder", "main.ui"),
                             domain="software-properties")
 
     if parent:
-      self.window_main.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_DIALOG)
+      self.window_main.set_type_hint(Gdk.WindowTypeHint.DIALOG)
       self.window_main.show()
       try:
         self.window_main.set_transient_for(parent)
@@ -100,12 +101,15 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGtkbuilderApp):
     # If externally called, reparent to external application.
     self.options = options
     if options and options.toplevel != None:
-      self.window_main.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_DIALOG)
+      self.window_main.set_type_hint(Gdk.WindowTypeHint.DIALOG)
       self.window_main.show()
-      toplevel = gtk.gdk.window_foreign_new(int(options.toplevel))
+      try:
+        toplevel = Gdk.window_foreign_new(int(options.toplevel))
+      except AttributeError:
+        toplevel = None
       if (toplevel):
         try:
-          self.window_main.window.set_transient_for(toplevel)
+          self.window_main.set_transient_for(toplevel)
         except: 
           pass
     if options and options.open_tab:
@@ -147,8 +151,8 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGtkbuilderApp):
 
   def update_interface(self):
     " abstract interface to keep the UI alive "
-    while gtk.events_pending():
-      gtk.main_iteration()
+    while Gtk.events_pending():
+      Gtk.main_iteration()
 
   def init_popcon(self):
     """ If popcon is enabled show the statistics tab and an explanation
@@ -167,8 +171,8 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGtkbuilderApp):
 
   def init_auto_update(self):
     """ Set up the widgets that allow to configure the update automation """
-    self.combobox_update_interval = gtk.combo_box_new_text()
-    self.hbox_check_for_updates.pack_start(self.combobox_update_interval)
+    self.combobox_update_interval = Gtk.ComboBoxText.new_with_entry()
+    self.hbox_check_for_updates.pack_start(self.combobox_update_interval, True, True, 0)
     self.combobox_update_interval.show()
     
     # this maps the key (combo-box-index) to the auto-update-interval value
@@ -185,7 +189,7 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGtkbuilderApp):
     self.combobox_update_interval.append_text(_("Weekly"))
     self.combobox_update_interval.append_text(_("Every two weeks"))
 
-    model_check_interval = gtk.ListStore(gobject.TYPE_STRING,
+    model_check_interval = Gtk.ListStore(gobject.TYPE_STRING,
                                          gobject.TYPE_INT)
     update_days = self.get_update_interval()
 
@@ -261,9 +265,9 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGtkbuilderApp):
 
     # TRANS: %s stands for the distribution name e.g. Debian or Ubuntu
     self.label_updates.set_label("<b>%s</b>" % (_("%s updates") %\
-                                                self.distro.id))
+                                                self.distro.id.encode('UTF-8')))
     # TRANS: %s stands for the distribution name e.g. Debian or Ubuntu
-    self.label_dist_name.set_label(_("%s Software") % self.distro.id)
+    self.label_dist_name.set_label(_("%s Software") % self.distro.id.encode('UTF-8'))
 
 
     self.handlers.append((self.checkbutton_source_code,
@@ -278,7 +282,7 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGtkbuilderApp):
         #              first %s is the description of the component
         #              second %s is the code name of the comp, eg main, universe
         label = _("%s (%s)") % (comp.get_description(), comp.name)
-        checkbox = gtk.CheckButton(label)
+        checkbox = Gtk.CheckButton(label)
 
         checkbox.comp = comp
         # setup the callback and show the checkbutton
@@ -295,7 +299,11 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGtkbuilderApp):
     if len(self.distro.source_template.children) < 1:
         self.frame_children.hide()
     for template in self.distro.source_template.children:
-        checkbox = gtk.CheckButton(label="%s (%s)" % (template.description,
+        # Do not show source entries in there
+        if template.type == "deb-src":
+            continue
+
+        checkbox = Gtk.CheckButton(label="%s (%s)" % (template.description,
                                                       template.name))
         checkbox.template = template
         self.handlers.append((checkbox,
@@ -307,13 +315,13 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGtkbuilderApp):
 
 
     # setup the server chooser
-    cell = gtk.CellRendererText()
+    cell = Gtk.CellRendererText()
     self.combobox_server.pack_start(cell, True)
     self.combobox_server.add_attribute(cell, 'text', 0)
     self.handlers.append((self.combobox_server,
                           self.combobox_server.connect("changed",
                               self.on_combobox_server_changed)))
-    server_store = gtk.ListStore(gobject.TYPE_STRING,
+    server_store = Gtk.ListStore(gobject.TYPE_STRING,
                                  gobject.TYPE_STRING,
                                  gobject.TYPE_BOOLEAN)
     self.combobox_server.set_model(server_store)
@@ -421,7 +429,7 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGtkbuilderApp):
     '''Call the backend to set the update automation level to the given 
        value'''
     if widget.get_active() == True:
-        self.vbox_auto_updates.foreach(lambda b: b.set_inconsistent(False))
+        self.vbox_auto_updates.foreach(lambda b, d: b.set_inconsistent(False), None)
         SoftwareProperties.set_update_automation_level(self, state)
     self.set_modified_config()
 
@@ -536,13 +544,13 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGtkbuilderApp):
     # STORE_SOURCE - the source entry object
     # STORE_SEPARATOR - if the entry is a separator
     # STORE_VISIBLE - if the entry is shown or hidden
-    self.cdrom_store = gtk.ListStore(gobject.TYPE_BOOLEAN, 
+    self.cdrom_store = Gtk.ListStore(gobject.TYPE_BOOLEAN, 
                                      gobject.TYPE_STRING,
                                      gobject.TYPE_PYOBJECT,
                                      gobject.TYPE_BOOLEAN,
                                      gobject.TYPE_BOOLEAN)
     self.treeview_cdroms.set_model(self.cdrom_store)
-    self.source_store = gtk.ListStore(gobject.TYPE_BOOLEAN, 
+    self.source_store = Gtk.ListStore(gobject.TYPE_BOOLEAN, 
                                       gobject.TYPE_STRING,
                                       gobject.TYPE_PYOBJECT,
                                       gobject.TYPE_BOOLEAN,
@@ -551,51 +559,55 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGtkbuilderApp):
     self.treeview_sources.set_row_separator_func(self.is_separator,
                                                  STORE_SEPARATOR)
 
-    cell_desc = gtk.CellRendererText()
+    cell_desc = Gtk.CellRendererText()
     cell_desc.set_property("xpad", 2)
     cell_desc.set_property("ypad", 2)
-    col_desc = gtk.TreeViewColumn(_("Software Sources"), cell_desc,
+    col_desc = Gtk.TreeViewColumn(_("Software Sources"), cell_desc,
                                   markup=COLUMN_DESC)
     col_desc.set_max_width(1000)
 
-    cell_toggle = gtk.CellRendererToggle()
+    cell_toggle = Gtk.CellRendererToggle()
     cell_toggle.set_property("xpad", 2)
     cell_toggle.set_property("ypad", 2)
     self.handlers.append([cell_toggle,
                           cell_toggle.connect('toggled', 
                                               self.on_isv_source_toggled, 
                                               self.cdrom_store)])
-    col_active = gtk.TreeViewColumn(_("Active"), cell_toggle,
+    col_active = Gtk.TreeViewColumn(_("Active"), cell_toggle,
                                     active=COLUMN_ACTIVE)
 
     self.treeview_cdroms.append_column(col_active)
     self.treeview_cdroms.append_column(col_desc)
 
-    cell_desc = gtk.CellRendererText()
+    cell_desc = Gtk.CellRendererText()
     cell_desc.set_property("xpad", 2)
     cell_desc.set_property("ypad", 2)
-    col_desc = gtk.TreeViewColumn(_("Software Sources"), cell_desc,
+    col_desc = Gtk.TreeViewColumn(_("Software Sources"), cell_desc,
                                   markup=COLUMN_DESC)
     col_desc.set_max_width(1000)
 
-    cell_toggle = gtk.CellRendererToggle()
+    cell_toggle = Gtk.CellRendererToggle()
     cell_toggle.set_property("xpad", 2)
     cell_toggle.set_property("ypad", 2)
     self.handlers.append([cell_toggle,
                           cell_toggle.connect('toggled', 
                                               self.on_isv_source_toggled, 
                                               self.source_store)])
-    col_active = gtk.TreeViewColumn(_("Active"), cell_toggle,
+    col_active = Gtk.TreeViewColumn(_("Active"), cell_toggle,
                                     active=COLUMN_ACTIVE)
 
     self.treeview_sources.append_column(col_active)
     self.treeview_sources.append_column(col_desc)
     # drag and drop support for sources.list
-    self.treeview_sources.drag_dest_set(gtk.DEST_DEFAULT_ALL,
-                                        [('text/uri-list',0, 0)],
-                                        gtk.gdk.ACTION_COPY)
-    self.treeview_sources.connect("drag_data_received",
-                                  self.on_sources_drag_data_received)
+    try:
+        Gtk.drag_dest_set(self.treeview_sources, Gtk.DestDefaults.ALL,
+                                            [Gtk.TargetEntry.new('text/uri-list', 0, 0)],
+                                            Gdk.DragAction.COPY)
+        self.treeview_sources.connect("drag_data_received",
+                                      self.on_sources_drag_data_received)
+    except AttributeError:
+        # does not work with Gtk2/GI
+        pass
 
   def on_isv_source_activate(self, treeview, path, column):
     """Open the edit dialog if a channel was double clicked"""
@@ -621,22 +633,27 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGtkbuilderApp):
 
   def init_keys(self):
     """Setup the user interface parts needed for the key handling"""
-    self.keys_store = gtk.ListStore(str)
+    self.keys_store = Gtk.ListStore(str)
     self.treeview_auth.set_model(self.keys_store)
-    tr = gtk.CellRendererText()
-    keys_col = gtk.TreeViewColumn("Key", tr, text=0)
+    tr = Gtk.CellRendererText()
+    keys_col = Gtk.TreeViewColumn("Key", tr, text=0)
     self.treeview_auth.append_column(keys_col)
-    self.treeview_auth.enable_model_drag_dest(
-      [('text/plain', 0, 0)], gtk.gdk.ACTION_COPY)
-    self.treeview_auth.connect("drag_data_received",
-                                  self.on_auth_drag_data_received)
+    try:
+        self.treeview_auth.enable_model_drag_dest(
+          [('text/plain', 0, 0)], Gdk.DragAction.COPY)
+        self.treeview_auth.connect("drag_data_received",
+                                      self.on_auth_drag_data_received)
+    except AttributeError:
+        # Does not work with GTK 2/GI
+        pass
+
     self.treeview_auth.connect("button-press-event",
                                self.show_auth_context_menu)
 
   def show_auth_context_menu(self, widget, event):
-    if event.type == gtk.gdk.BUTTON_PRESS and event.button == 3:
-      menu = gtk.Menu()
-      item_paste = gtk.MenuItem(_("_Add key from paste data"))
+    if event.type == Gdk.EventType.BUTTON_PRESS and event.button == 3:
+      menu = Gtk.Menu()
+      item_paste = Gtk.MenuItem(label=_("_Add key from paste data"))
       item_paste.connect("activate", self.on_auth_add_key_from_paste)
       menu.add(item_paste)
       menu.popup(None, None, None, 0, event.time)
@@ -644,7 +661,7 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGtkbuilderApp):
       return True
 
   def on_auth_add_key_from_paste(self, widget):
-    keydata = gtk.Clipboard().wait_for_text()
+    keydata = Gtk.Clipboard().wait_for_text()
     if not keydata:
       return
     if not self.add_key_from_data(keydata):
@@ -744,7 +761,7 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGtkbuilderApp):
           self.combobox_update_interval.set_active(i)
       value = self.combobox_interval_mapping[i]
       # A little hack to re-set the former selected update automation level
-      self.vbox_auto_updates.foreach(lambda b: b.toggled())
+      self.vbox_auto_updates.foreach(lambda b, d: b.toggled(), None)
     else:
       self.combobox_update_interval.set_sensitive(False)
       self.vbox_auto_updates.set_sensitive(False)
@@ -770,7 +787,7 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGtkbuilderApp):
     source_entry = model.get_value(iter, LIST_ENTRY_OBJ)
     dialog = DialogEdit(self.window_main, self.sourceslist,
                         source_entry, self.datadir)
-    if dialog.run() == gtk.RESPONSE_OK:
+    if dialog.run() == Gtk.ResponseType.OK:
         self.set_modified_sourceslist()
 
   # FIXME:outstanding from merge
@@ -812,17 +829,17 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGtkbuilderApp):
   def add_key_clicked(self, widget):
     """Provide a file chooser that allows to add the gnupg of a trusted
        software vendor"""
-    chooser = gtk.FileChooserDialog(title=_("Import key"),
+    chooser = Gtk.FileChooserDialog(title=_("Import key"),
                                     parent=self.window_main,
-                                    buttons=(gtk.STOCK_CANCEL,
-                                             gtk.RESPONSE_REJECT,
-                                             gtk.STOCK_OK,gtk.RESPONSE_ACCEPT))
+                                    buttons=(Gtk.STOCK_CANCEL,
+                                             Gtk.ResponseType.REJECT,
+                                             Gtk.STOCK_OK,Gtk.ResponseType.ACCEPT))
     if "SUDO_USER" in os.environ:
         home = os.path.expanduser("~%s" % os.environ["SUDO_USER"])
         chooser.set_current_folder(home)
     res = chooser.run()
     chooser.hide()
-    if res == gtk.RESPONSE_ACCEPT:
+    if res == Gtk.ResponseType.ACCEPT:
       if not self.add_key(chooser.get_filename()):
         error(self.window_main,
               _("Error importing selected file"),
