@@ -23,7 +23,9 @@ import apt_pkg
 import re
 import subprocess
 from threading import Thread
-import pycurl
+import urllib
+from urllib2 import urlopen, Request, URLError
+from urlparse import urlparse
 
 def encode(s):
     return re.sub("[^a-zA-Z0-9_-]","_", s)
@@ -49,12 +51,6 @@ def expand_ppa_line(abrev, distro_codename):
         sourceslistd, encode(ppa_owner), encode(ppa_name), distro_codename)
     return (line, filename)
 
-class CurlCallback:
-    def __init__(self):
-        self.contents = ''
-
-    def body_callback(self, buf):
-        self.contents = self.contents + buf
 
 class AddPPASigningKeyThread(Thread):
     " thread class for adding the signing key in the background "
@@ -78,16 +74,9 @@ class AddPPASigningKeyThread(Thread):
         try:
             # we ask for a JSON structure from lp_page, we could use
             # simplejson, but the format is simple enough for the regexp
-            callback = CurlCallback()
-            curl = pycurl.Curl()
-            curl.setopt(pycurl.SSL_VERIFYPEER, 1)
-            curl.setopt(pycurl.SSL_VERIFYHOST, 2)
-            curl.setopt(pycurl.WRITEFUNCTION, callback.body_callback)
-            curl.setopt(pycurl.URL, str(lp_url))
-            curl.setopt(pycurl.HTTPHEADER, ["Accept: application/json"])
-            curl.perform()
-            curl.close()
-            lp_page = callback.contents
+            req =  Request(lp_url)
+            req.add_header("Accept","application/json")
+            lp_page = urlopen(req).read()
             #print lp_page
             signing_key_fingerprint = re.findall(
                 '\"signing_key_fingerprint\": \"(\w*)\"', lp_page)
@@ -96,11 +85,12 @@ class AddPPASigningKeyThread(Thread):
             # FIXME: this needs to go - elmo says the keyserver will not handle
             #        the load
             res = subprocess.call(
-                ["apt-key", "adv", "--keyserver", "keyserver.ubuntu.com",
+                ["apt-key", "adv", 
+                 "--keyserver", "hkp://keyserver.ubuntu.com:80/",
                  "--recv", signing_key_fingerprint[0]])
             return (res == 0)
-        except pycurl.error, e:
-            print "Error reading %s: %s" % (lp_url, e[1])
+        except URLError, e:
+            print "Error reading %s: %s" % (lp_url, e)
             return False
         except IOError, e:
             print "Error: can't find signing_key_fingerprint at %s" % lp_url
